@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,9 @@
  * @short_description: User and window system events
  *
  * Windowing events handled by Clutter.
+ *
+ * The events usually come from the windowing backend, but can also
+ * be synthesized by Clutter itself or by the application code.
  */
 
 /**
@@ -94,12 +97,17 @@ clutter_event_get_state (ClutterEvent *event)
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
       return event->key.modifier_state;
+
     case CLUTTER_BUTTON_PRESS:
+    case CLUTTER_BUTTON_RELEASE:
       return event->button.modifier_state;
+
     case CLUTTER_MOTION:
       return event->motion.modifier_state;
+
     case CLUTTER_SCROLL:
       return event->scroll.modifier_state;
+
     default:
       break;
     }
@@ -119,10 +127,10 @@ clutter_event_get_state (ClutterEvent *event)
  */
 void
 clutter_event_get_coords (ClutterEvent *event,
-                          gint         *x,
-                          gint         *y)
+                          gfloat       *x,
+                          gfloat       *y)
 {
-  gint event_x, event_y;
+  gfloat event_x, event_y;
 
   g_return_if_fail (event != NULL);
 
@@ -137,18 +145,25 @@ clutter_event_get_coords (ClutterEvent *event,
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
     case CLUTTER_DELETE:
+      break;
+
     case CLUTTER_ENTER:
     case CLUTTER_LEAVE:
+      event_x = event->crossing.x;
+      event_y = event->crossing.y;
       break;
+
     case CLUTTER_BUTTON_PRESS:
     case CLUTTER_BUTTON_RELEASE:
       event_x = event->button.x;
       event_y = event->button.y;
       break;
+
     case CLUTTER_MOTION:
       event_x = event->motion.x;
       event_y = event->motion.y;
       break;
+
     case CLUTTER_SCROLL:
       event_x = event->scroll.x;
       event_y = event->scroll.y;
@@ -173,7 +188,7 @@ clutter_event_get_coords (ClutterEvent *event,
  *
  * Since: 0.6
  */
-ClutterActor*
+ClutterActor *
 clutter_event_get_source (ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
@@ -186,13 +201,13 @@ clutter_event_get_source (ClutterEvent *event)
  * @event: a #ClutterEvent
  *
  * Retrieves the source #ClutterStage the event originated for, or
- * NULL if the event has no stage.
+ * %NULL if the event has no stage.
  *
  * Return value: (transfer none): a #ClutterStage
  *
  * Since: 0.8
  */
-ClutterStage*
+ClutterStage *
 clutter_event_get_stage (ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
@@ -201,74 +216,168 @@ clutter_event_get_stage (ClutterEvent *event)
 }
 
 /**
- * clutter_button_event_button:
- * @buttev: a #ClutterButtonEvent
+ * clutter_event_get_flags:
+ * @event: a #ClutterEvent
  *
- * Retrieve the button number of the event.
+ * Retrieves the #ClutterEventFlags of @event
  *
- * Return value: the button number.
+ * Return value: the event flags
  *
- * Since: 0.4
+ * Since: 1.0
+ */
+ClutterEventFlags
+clutter_event_get_flags (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, CLUTTER_EVENT_NONE);
+
+  return event->any.flags;
+}
+
+/**
+ * clutter_event_get_related:
+ * @event: a #ClutterEvent of type %CLUTTER_ENTER or of
+ *   type %CLUTTER_LEAVE
+ *
+ * Retrieves the related actor of a crossing event.
+ *
+ * Return value: (transfer none): the related #ClutterActor, or %NULL
+ *
+ * Since: 1.0
+ */
+ClutterActor *
+clutter_event_get_related (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, NULL);
+  g_return_val_if_fail (event->type == CLUTTER_ENTER ||
+                        event->type == CLUTTER_LEAVE, NULL);
+
+  return event->crossing.related;
+}
+
+/**
+ * clutter_event_get_scroll_direction:
+ * @event: a #ClutterEvent of type %CLUTTER_SCROLL
+ *
+ * Retrieves the direction of the scrolling of @event
+ *
+ * Return value: the scrolling direction
+ *
+ * Since: 1.0
+ */
+ClutterScrollDirection
+clutter_event_get_scroll_direction (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, CLUTTER_SCROLL_UP);
+  g_return_val_if_fail (event->type == CLUTTER_SCROLL, CLUTTER_SCROLL_UP);
+
+  return event->scroll.direction;
+}
+
+/**
+ * clutter_event_get_button:
+ * @event: a #ClutterEvent of type %CLUTTER_BUTTON_PRESS or
+ *   of type %CLUTTER_BUTTON_RELEASE
+ *
+ * Retrieves the button number of @event
+ *
+ * Return value: the button number
+ *
+ * Since: 1.0
  */
 guint32
-clutter_button_event_button (ClutterButtonEvent *buttev)
+clutter_event_get_button (ClutterEvent *event)
 {
-  g_return_val_if_fail (buttev != NULL, 0);
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
+                        event->type == CLUTTER_BUTTON_RELEASE, 0);
 
-  return buttev->button;
+  return event->button.button;
+}
+
+/**
+ * clutter_event_get_click_count:
+ * @event: a #ClutterEvent of type %CLUTTER_BUTTON_PRESS or
+ *   of type %CLUTTER_BUTTON_RELEASE
+ *
+ * Retrieves the number of clicks of @event
+ *
+ * Return value: the click count
+ *
+ * Since: 1.0
+ */
+guint32
+clutter_event_get_click_count (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
+                        event->type == CLUTTER_BUTTON_RELEASE, 0);
+
+  return event->button.click_count;
 }
 
 /* keys */
 
 /**
- * clutter_key_event_symbol:
- * @keyev: A #ClutterKeyEvent
+ * clutter_event_get_key_symbol:
+ * @event: a #ClutterEvent of type %CLUTTER_KEY_PRESS or
+ *   of type %CLUTTER_KEY_RELEASE
  *
- * Retrieves the symbol of the key that caused @keyev.
+ * Retrieves the key symbol of @event
  *
- * Return value: The key symbol representing the key
+ * Return value: the key symbol representing the key
+ *
+ * Since: 1.0
  */
 guint
-clutter_key_event_symbol (ClutterKeyEvent *keyev)
+clutter_event_get_key_symbol (ClutterEvent *event)
 {
-  g_return_val_if_fail (keyev != NULL, 0);
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
+                        event->type == CLUTTER_KEY_RELEASE, 0);
 
-  return keyev->keyval;
+  return event->key.keyval;
 }
 
 /**
- * clutter_key_event_code:
- * @keyev: A #ClutterKeyEvent
+ * clutter_event_get_key_code:
+ * @event: a #ClutterEvent of type %CLUTTER_KEY_PRESS or
+ *    of type %CLUTTER_KEY_RELEASE
  *
- * Retrieves the keycode of the key that caused @keyev.
+ * Retrieves the keycode of the key that caused @event
  *
  * Return value: The keycode representing the key
+ *
+ * Since: 1.0
  */
 guint16
-clutter_key_event_code (ClutterKeyEvent *keyev)
+clutter_event_get_key_code (ClutterEvent *event)
 {
-  g_return_val_if_fail (keyev != NULL, 0);
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
+                        event->type == CLUTTER_KEY_RELEASE, 0);
 
-  return keyev->hardware_keycode;
+  return event->key.hardware_keycode;
 }
 
 /**
- * clutter_key_event_unicode:
- * @keyev: A #ClutterKeyEvent
+ * clutter_event_get_key_unicode:
+ * @event: A #ClutterKeyEvent
  *
  * Retrieves the unicode value for the key that caused @keyev.
  *
  * Return value: The unicode value representing the key
  */
 guint32
-clutter_key_event_unicode (ClutterKeyEvent *keyev)
+clutter_event_get_key_unicode (ClutterEvent *event)
 {
-  g_return_val_if_fail (keyev != NULL, 0);
+  g_return_val_if_fail (event != NULL, 0);
+  g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
+                        event->type == CLUTTER_KEY_RELEASE, 0);
 
-  if (keyev->unicode_value)
-    return keyev->unicode_value;
+  if (event->key.unicode_value)
+    return event->key.unicode_value;
   else
-    return clutter_keysym_to_unicode (keyev->keyval);
+    return clutter_keysym_to_unicode (event->key.keyval);
 }
 
 /**
@@ -278,9 +387,9 @@ clutter_key_event_unicode (ClutterKeyEvent *keyev)
  * Convert from a Clutter key symbol to the corresponding ISO10646 (Unicode)
  * character.
  * 
- * Return value: the corresponding unicode character, or 0 if there
- *               is no corresponding character.
- **/
+ * Return value: a Unicode character, or 0 if there  is no corresponding
+ *   character.
+ */
 guint32
 clutter_keysym_to_unicode (guint keyval)
 {
@@ -299,17 +408,20 @@ clutter_keysym_to_unicode (guint keyval)
     return keyval & 0x00ffffff;
 
   /* binary search in table */
-  while (max >= min) {
-    mid = (min + max) / 2;
-    if (clutter_keysym_to_unicode_tab[mid].keysym < keyval)
-      min = mid + 1;
-    else if (clutter_keysym_to_unicode_tab[mid].keysym > keyval)
-      max = mid - 1;
-    else {
-      /* found it */
-      return clutter_keysym_to_unicode_tab[mid].ucs;
+  while (max >= min)
+    {
+      mid = (min + max) / 2;
+
+      if (clutter_keysym_to_unicode_tab[mid].keysym < keyval)
+        min = mid + 1;
+      else if (clutter_keysym_to_unicode_tab[mid].keysym > keyval)
+        max = mid - 1;
+      else
+        {
+          /* found it */
+          return clutter_keysym_to_unicode_tab[mid].ucs;
+        }
     }
-  }
  
   /* No matching Unicode value found */
   return 0;
@@ -318,12 +430,12 @@ clutter_keysym_to_unicode (guint keyval)
 /**
  * clutter_event_get_device_id:
  * @event: a clutter event 
- * 
+ *
  * Retrieves the events device id if set.
- * 
+ *
  * Return value: A unique identifier for the device or -1 if the event has
- *               no specific device set.
- **/
+ *   no specific device set.
+ */
 gint
 clutter_event_get_device_id (ClutterEvent *event)
 {
@@ -353,15 +465,122 @@ clutter_event_get_device_id (ClutterEvent *event)
       break;
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
+      device = event->scroll.device;
       break;
     }
 
-  if (device)
+  if (device != NULL)
     return device->id;
   else
     return -1;
 }
 
+/**
+ * clutter_event_get_device_type:
+ * @event: a #ClutterEvent
+ *
+ * Retrieves the type of the device for @event
+ *
+ * Return value: the #ClutterInputDeviceType for the device, if
+ *   any is set
+ *
+ * Since: 1.0
+ */
+ClutterInputDeviceType
+clutter_event_get_device_type (ClutterEvent *event)
+{
+  ClutterInputDevice *device = NULL;
+
+  g_return_val_if_fail (event != NULL, CLUTTER_POINTER_DEVICE);
+
+  switch (event->type)
+    {
+    case CLUTTER_NOTHING:
+    case CLUTTER_STAGE_STATE:
+    case CLUTTER_DESTROY_NOTIFY:
+    case CLUTTER_CLIENT_MESSAGE:
+    case CLUTTER_DELETE:
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      break;
+
+    case CLUTTER_BUTTON_PRESS:
+    case CLUTTER_BUTTON_RELEASE:
+      device = event->button.device;
+      break;
+
+    case CLUTTER_MOTION:
+      device = event->motion.device;
+      break;
+
+    case CLUTTER_SCROLL:
+      device = event->scroll.device;
+      break;
+
+    case CLUTTER_KEY_PRESS:
+    case CLUTTER_KEY_RELEASE:
+      device = event->scroll.device;
+      break;
+    }
+
+  if (device != NULL)
+    return device->device_type;
+  else
+    return CLUTTER_POINTER_DEVICE;
+}
+
+/**
+ * clutter_event_get_device:
+ * @event: a #ClutterEvent
+ *
+ * Retrieves the #ClutterInputDevice for the event.
+ *
+ * The #ClutterInputDevice structure is completely opaque and should
+ * be cast to the platform-specific implementation.
+ *
+ * Return value: the #ClutterInputDevice or %NULL
+ *
+ * Since: 1.0
+ */
+ClutterInputDevice *
+clutter_event_get_device (ClutterEvent *event)
+{
+  ClutterInputDevice *device = NULL;
+
+  g_return_val_if_fail (event != NULL, NULL);
+
+  switch (event->type)
+    {
+    case CLUTTER_NOTHING:
+    case CLUTTER_STAGE_STATE:
+    case CLUTTER_DESTROY_NOTIFY:
+    case CLUTTER_CLIENT_MESSAGE:
+    case CLUTTER_DELETE:
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      break;
+
+    case CLUTTER_BUTTON_PRESS:
+    case CLUTTER_BUTTON_RELEASE:
+      device = event->button.device;
+      break;
+
+    case CLUTTER_MOTION:
+      device = event->motion.device;
+      break;
+
+    case CLUTTER_SCROLL:
+      device = event->scroll.device;
+      break;
+
+    case CLUTTER_KEY_PRESS:
+    case CLUTTER_KEY_RELEASE:
+      device = event->scroll.device;
+      break;
+    }
+
+  return device;
+}
 
 GType
 clutter_event_get_type (void)
@@ -424,10 +643,8 @@ clutter_event_copy (ClutterEvent *event)
 void
 clutter_event_free (ClutterEvent *event)
 {
-  if (G_LIKELY (event))
-    {
-      g_slice_free (ClutterEvent, event);
-    }
+  if (G_LIKELY (event != NULL))
+    g_slice_free (ClutterEvent, event);
 }
 
 /**
@@ -443,7 +660,7 @@ clutter_event_free (ClutterEvent *event)
 ClutterEvent *
 clutter_event_get (void)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   if (!context->events_queue)
     return NULL;
@@ -467,7 +684,7 @@ clutter_event_get (void)
 ClutterEvent *
 clutter_event_peek (void)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   g_return_val_if_fail (context != NULL, NULL);
   
@@ -484,18 +701,18 @@ clutter_event_peek (void)
  * clutter_event_put:
  * @event: a #ClutterEvent
  *
- * Puts a copy of the event on the back of the event queue. The event will have
- * the #CLUTTER_EVENT_FLAG_SYNTHETIC flag set. If the source is set event
- * signals will be emitted for this source and capture/bubbling for it's
- * ancestors. If the source is not set it will be generated by picking or use
- * the actor that currently has keyboard focus.
+ * Puts a copy of the event on the back of the event queue. The event will
+ * have the %CLUTTER_EVENT_FLAG_SYNTHETIC flag set. If the source is set
+ * event signals will be emitted for this source and capture/bubbling for
+ * its ancestors. If the source is not set it will be generated by picking
+ * or use the actor that currently has keyboard focus
  *
  * Since: 0.6
  */
 void
 clutter_event_put (ClutterEvent *event)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
   ClutterEvent       *event_copy;
 
   /* FIXME: check queue is valid */
@@ -519,7 +736,7 @@ clutter_event_put (ClutterEvent *event)
 gboolean
 clutter_events_pending (void)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   g_return_val_if_fail (context != NULL, FALSE);
 
@@ -542,7 +759,7 @@ clutter_events_pending (void)
 guint32
 clutter_get_current_event_time (void)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   g_return_val_if_fail (context != NULL, FALSE);
 
@@ -551,3 +768,26 @@ clutter_get_current_event_time (void)
 
   return CLUTTER_CURRENT_TIME;
 }
+
+/**
+ * clutter_get_current_event:
+ *
+ * If an event is currently being processed, return that event.
+ * This function is intended to be used to access event state
+ * that might not be exposed by higher-level widgets.  For
+ * example, to get the key modifier state from a Button 'clicked'
+ * event.
+ *
+ * Return value: (transfer none): The current ClutterEvent, or %NULL if none
+ *
+ * Since: 1.2
+ */
+G_CONST_RETURN ClutterEvent *
+clutter_get_current_event (void)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  g_return_val_if_fail (context != NULL, NULL);
+
+  return context->current_event;
+}
