@@ -82,8 +82,15 @@ clutter_backend_glx_handle_event (ClutterBackendX11 *backend_x11,
 
       if (stage_x11->xwin == swap_complete_event->drawable)
         {
-          g_assert (stage_glx->pending_swaps);
-          stage_glx->pending_swaps--;
+	  /* Early versions of the swap_event implementation in Mesa
+	   * deliver BufferSwapComplete event when not selected for,
+	   * so if we get a swap event we aren't expecting, just ignore it.
+	   *
+	   * https://bugs.freedesktop.org/show_bug.cgi?id=27962
+	   */
+          if (stage_glx->pending_swaps > 0)
+            stage_glx->pending_swaps--;
+
           return TRUE;
         }
     }
