@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 /**
@@ -44,6 +44,7 @@
 #include "clutter-x11-texture-pixmap.h"
 #include "clutter-x11.h"
 #include "clutter-backend-x11.h"
+#include "clutter-private.h"
 
 #include "cogl/cogl.h"
 
@@ -74,6 +75,7 @@ enum
 enum
 {
   UPDATE_AREA,
+  QUEUE_DAMAGE_REDRAW,
   /* FIXME: Pixmap lost signal? */
   LAST_SIGNAL
 };
@@ -255,60 +257,68 @@ failed_image_create:
   return FALSE;
 }
 
+static void
+check_for_pixmap_damage (ClutterX11TexturePixmap *texture)
+{
+  ClutterX11TexturePixmapPrivate *priv = texture->priv;
+  Display *dpy;
+  XserverRegion parts;
+  int i, r_count;
+  XRectangle *r_damage;
+  XRectangle r_bounds;
+
+  clutter_x11_trap_x_errors ();
+
+  /*
+   * Retrieve the damaged region and break it down into individual
+   * rectangles so we do not have to update the whole shebang.
+   */
+  dpy = clutter_x11_get_default_display();
+  parts = XFixesCreateRegion (dpy, 0, 0);
+  XDamageSubtract (dpy, priv->damage, None, parts);
+
+  r_damage = XFixesFetchRegionAndBounds (dpy,
+                                         parts,
+                                         &r_count,
+                                         &r_bounds);
+
+  clutter_x11_untrap_x_errors ();
+
+  if (r_damage)
+    {
+      for (i = 0; i < r_count; ++i)
+        clutter_x11_texture_pixmap_update_area (texture,
+                                                r_damage[i].x,
+                                                r_damage[i].y,
+                                                r_damage[i].width,
+                                                r_damage[i].height);
+      XFree (r_damage);
+    }
+
+  XFixesDestroyRegion (dpy, parts);
+}
+
 static ClutterX11FilterReturn
 on_x_event_filter (XEvent *xev, ClutterEvent *cev, gpointer data)
 {
   ClutterX11TexturePixmap        *texture;
   ClutterX11TexturePixmapPrivate *priv;
-  Display                        *dpy;
 
   texture = CLUTTER_X11_TEXTURE_PIXMAP (data);
 
   g_return_val_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture), \
                         CLUTTER_X11_FILTER_CONTINUE);
 
-  dpy = clutter_x11_get_default_display();
   priv = texture->priv;
 
   if (xev->type == _damage_event_base + XDamageNotify)
     {
-      XserverRegion  parts;
-      gint           i, r_count;
-      XRectangle    *r_damage;
-      XRectangle     r_bounds;
       XDamageNotifyEvent *dev = (XDamageNotifyEvent*)xev;
 
       if (dev->drawable != priv->damage_drawable)
         return CLUTTER_X11_FILTER_CONTINUE;
 
-
-      clutter_x11_trap_x_errors ();
-      /*
-       * Retrieve the damaged region and break it down into individual
-       * rectangles so we do not have to update the whole shebang.
-       */
-      parts = XFixesCreateRegion (dpy, 0, 0);
-      XDamageSubtract (dpy, priv->damage, None, parts);
-
-      r_damage = XFixesFetchRegionAndBounds (dpy,
-                                             parts,
-                                             &r_count,
-                                             &r_bounds);
-
-      clutter_x11_untrap_x_errors ();
-
-      if (r_damage)
-        {
-          for (i = 0; i < r_count; ++i)
-            clutter_x11_texture_pixmap_update_area (texture,
-                                                    r_damage[i].x,
-                                                    r_damage[i].y,
-                                                    r_damage[i].width,
-                                                    r_damage[i].height);
-          XFree (r_damage);
-        }
-
-      XFixesDestroyRegion (dpy, parts);
+      check_for_pixmap_damage (texture);
     }
 
   return  CLUTTER_X11_FILTER_CONTINUE;
@@ -404,6 +414,58 @@ free_damage_resources (ClutterX11TexturePixmap *texture)
     }
 }
 
+static void
+clutter_x11_texture_pixmap_real_queue_damage_redraw (
+                                              ClutterX11TexturePixmap *texture,
+                                              gint x,
+                                              gint y,
+                                              gint width,
+                                              gint height)
+{
+  ClutterActor    *self = CLUTTER_ACTOR (texture);
+  ClutterActorBox  allocation;
+  guint	           pixmap_width = 0;
+  guint            pixmap_height = 0;
+  float            scale_x;
+  float            scale_y;
+  ClutterActorBox  clip;
+
+  /* NB: clutter_actor_queue_clipped_redraw expects a box in the actor's
+   * coordinate space so we need to convert from pixmap coordinates to
+   * actor coordinates...
+   */
+
+  /* Calling clutter_actor_get_allocation_box() is enormously expensive
+   * if the actor has an out-of-date allocation, since it triggers
+   * a full redraw. clutter_actor_queue_clipped_redraw() would redraw
+   * the whole stage anyways in that case, so just go ahead and do
+   * it here.
+   */
+  if (!_clutter_actor_has_allocation (self))
+    {
+      clutter_actor_queue_redraw (self);
+      return;
+    }
+
+  clutter_actor_get_allocation_box (self, &allocation);
+
+  g_object_get (self,
+                "pixmap-width",  &pixmap_width,
+                "pixmap-height", &pixmap_height,
+                NULL);
+
+  scale_x = (allocation.x2 - allocation.x1) / pixmap_width;
+  scale_y = (allocation.y2 - allocation.y1) / pixmap_height;
+
+  clip.x1 = x * scale_x;
+  clip.y1 = y * scale_y;
+  clip.x2 =  clip.x1 + width * scale_x;
+  clip.y2 = clip.y1 + height * scale_y;
+
+  _clutter_actor_queue_redraw_with_clip (self,
+                                         CLUTTER_REDRAW_CLIPPED_TO_BOX,
+                                         &clip);
+}
 
 static void
 clutter_x11_texture_pixmap_init (ClutterX11TexturePixmap *self)
@@ -729,6 +791,46 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
                     G_TYPE_INT,
                     G_TYPE_INT);
 
+  /**
+   * ClutterX11TexturePixmap::queue-damage-redraw
+   * @texture: the object which received the signal
+   * @x: The top left x position of the damage region
+   * @y: The top left y position of the damage region
+   * @width: The width of the damage region
+   * @height: The height of the damage region
+   *
+   * ::queue-damage-redraw is emitted to notify that some sub-region of the
+   * underlying pixmap has changed and you need to queue a
+   * corresponding redraw for the actor.
+   *
+   * The default handler will queue a clipped redraw in response to
+   * the damage, using the assumption that the pixmap is being painted
+   * to a rectangle covering the transformed allocation of the actor.
+   * If you sub-class and change the paint method so this isn't true
+   * then you must also provide your own damage signal handler to
+   * queue a redraw that blocks this default behaviour.
+   *
+   * Since: 1.2
+   */
+  signals[QUEUE_DAMAGE_REDRAW] =
+    g_signal_new (g_intern_static_string ("queue-damage-redraw"),
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL,
+                  clutter_marshal_VOID__INT_INT_INT_INT,
+                  G_TYPE_NONE, 4,
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  G_TYPE_INT);
+
+  g_signal_override_class_closure (signals[QUEUE_DAMAGE_REDRAW],
+                                   CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
+                                   g_cclosure_new ((GCallback) clutter_x11_texture_pixmap_real_queue_damage_redraw,
+                                                   NULL,
+                                                   NULL));
+
   default_backend = clutter_get_default_backend ();
 
   if (!CLUTTER_IS_BACKEND_X11 (default_backend))
@@ -941,10 +1043,13 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
 /**
  * clutter_x11_texture_pixmap_new:
  *
+ * Creates a new #ClutterX11TexturePixmap which can be used to display the
+ * contents of an X11 Pixmap inside a Clutter scene graph
+ *
  * Return value: A new #ClutterX11TexturePixmap
  *
  * Since: 0.8
- **/
+ */
 ClutterActor *
 clutter_x11_texture_pixmap_new (void)
 {
@@ -959,10 +1064,12 @@ clutter_x11_texture_pixmap_new (void)
  * clutter_x11_texture_pixmap_new_with_pixmap:
  * @pixmap: the X Pixmap to which this texture should be bound
  *
+ * Creates a new #ClutterX11TexturePixmap for @pixmap
+ *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X Pixmap
  *
  * Since: 0.8
- **/
+ */
 ClutterActor *
 clutter_x11_texture_pixmap_new_with_pixmap (Pixmap pixmap)
 {
@@ -979,6 +1086,8 @@ clutter_x11_texture_pixmap_new_with_pixmap (Pixmap pixmap)
  * clutter_x11_texture_pixmap_new_with_window:
  * @window: the X window to which this texture should be bound
  *
+ * Creates a new #ClutterX11TexturePixmap for @window
+ *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X window.
  *
  * Since: 0.8
@@ -1003,7 +1112,7 @@ clutter_x11_texture_pixmap_new_with_window (Window window)
  * Sets the X Pixmap to which the texture should be bound.
  *
  * Since: 0.8
- **/
+ */
 void
 clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
                                        Pixmap                   pixmap)
@@ -1125,16 +1234,17 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
  * clutter_x11_texture_pixmap_set_window:
  * @texture: the texture to bind
  * @window: the X window to which the texture should be bound
- * @automatic: TRUE is automatic window updates, FALSE for manual.
+ * @automatic: %TRUE for automatic window updates, %FALSE for manual.
  *
  * Sets up a suitable pixmap for the window, using the composite and damage
  * extensions if possible, and then calls
- * clutter_x11_texture_pixmap_set_pixmap(). If you want a window in a texture,
- * you probably want this function, or its older sister,
- * clutter_glx_texture_pixmap_set_window().
+ * clutter_x11_texture_pixmap_set_pixmap().
+ *
+ * If you want to display a window in a #ClutterTexture, you probably want
+ * this function, or its older sister, clutter_glx_texture_pixmap_set_window()
  *
  * Since: 0.8
- **/
+ */
 void
 clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
                                        Window                   window,
@@ -1226,7 +1336,7 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
  * pixmap's invalidation as the window changed size.
  *
  * Since: 0.8
- **/
+ */
 void
 clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
 {
@@ -1361,8 +1471,24 @@ clutter_x11_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
   g_signal_emit (texture, signals[UPDATE_AREA], 0, x, y, width, height);
+
+  /* The default handler for the "queue-damage-redraw" signal is
+   * clutter_x11_texture_pixmap_real_queue_damage_redraw which will queue a
+   * clipped redraw. */
+  g_signal_emit (texture, signals[QUEUE_DAMAGE_REDRAW],
+                 0, x, y, width, height);
 }
 
+/**
+ * clutter_x11_texture_pixmap_set_automatic:
+ * @texture: a #ClutterX11TexturePixmap
+ * @setting: %TRUE to enable automatic updates
+ *
+ * Enables or disables the automatic updates ot @texture in case the backing
+ * pixmap or window is damaged
+ *
+ * Since: 0.8
+ */
 void
 clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
                                           gboolean                 setting)
@@ -1387,3 +1513,4 @@ clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
   priv->automatic_updates = setting;
 
 }
+
